# -*- coding: utf-8 -*-
"""
License: BSD

(c) 2009      ::: www.CodeResort.com - BV Network AS (simon-code@bvnetwork.no)
"""

import os
import unittest
import urllib2

from tracrpc.tests import rpc_testenv

from tracrpc.api import IRPCProtocol

from trac.core import *
from trac.test import Mock

class ProtocolProviderTestCase(unittest.TestCase):

    def test_invalid_content_type(self):
        req = urllib2.Request(rpc_testenv.url_anon,
                    headers={'Content-Type': 'text/plain'})
        try:
            resp = urllib2.urlopen(req)
            self.fail("Expected urllib2.HTTPError")
        except urllib2.HTTPError, e:
            self.assertEquals(e.code, 415)
            self.assertEquals(e.msg, "Unsupported Media Type")
            self.assertEquals(e.fp.read(),
                "No protocol matching Content-Type 'text/plain' at path '/rpc'.")

    def test_valid_provider(self):
        # Confirm the request won't work before adding plugin
        req = urllib2.Request(rpc_testenv.url_anon,
                        headers={'Content-Type': 'application/x-tracrpc-test'})
        try:
            resp = urllib2.urlopen(req)
            self.fail("Expected urllib2.HTTPError")
        except urllib2.HTTPError, e:
            self.assertEquals(e.code, 415)
        # Make a new plugin 
        provider = os.path.join(rpc_testenv.tracdir, 'plugins', 'DummyProvider.py')
        open(provider, 'w').write(
            "from trac.core import *\n"
            "from tracrpc.api import *\n"
            "class DummyProvider(Component):\n"
            "    implements(IRPCProtocol)\n"
            "    def rpc_info(self):\n"
            "        yield ('TEST-RPC', 'No Docs!')\n"
            "    def rpc_match(self):\n"
            "        yield ('rpc', 'application/x-tracrpc-test')\n"
            "    def rpc_process(self, req, content_type):\n"
            "        response = 'Got a result!'\n"
            "        req.send_response(200)\n"
            "        req.send_header('Content-Type', content_type)\n"
            "        req.send_header('Content-Length', len(response))\n"
            "        req.end_headers()\n"
            "        req.write(response)\n")
        rpc_testenv.restart()
        try:
            req = urllib2.Request(rpc_testenv.url_anon,
                        headers={'Content-Type': 'application/x-tracrpc-test'})
            resp = urllib2.urlopen(req)
            self.assertEquals(200, resp.code)
            self.assertEquals("Got a result!", resp.read())
            self.assertEquals(resp.headers['Content-Type'],
                                                'application/x-tracrpc-test')
        finally:
            # Clean up so that provider don't affect further tests
            os.unlink(provider)
            rpc_testenv.restart()

def suite():
    return unittest.makeSuite(ProtocolProviderTestCase)

if __name__ == '__main__':
    unittest.main(defaultTest='suite')
